@@ -22,9 +22,9 @@ import (
 	"net/url"
 	"time"
 
-	"github.com/superseriousbusiness/activity/streams"
-	"github.com/superseriousbusiness/activity/streams/vocab"
-	"github.com/superseriousbusiness/gotosocial/internal/gtserror"
+	"code.superseriousbusiness.org/activity/streams"
+	"code.superseriousbusiness.org/activity/streams/vocab"
+	"code.superseriousbusiness.org/gotosocial/internal/gtserror"
 )
 
 // MustGet performs the given 'Get$Property(with) (T, error)' signature function, panicking on error.
@@ -135,7 +135,8 @@ func AppendBcc(with WithBcc, bcc ...*url.URL) {
 	}, bcc...)
 }
 
-// GetURL returns the IRIs contained in the URL property of 'with'.
+// GetURL returns IRIs contained
+// in the URL property of 'with'.
 func GetURL(with WithURL) []*url.URL {
 	urlProp := with.GetActivityStreamsUrl()
 	if urlProp == nil || urlProp.Len() == 0 {
@@ -144,9 +145,31 @@ func GetURL(with WithURL) []*url.URL {
 	urls := make([]*url.URL, 0, urlProp.Len())
 	for i := 0; i < urlProp.Len(); i++ {
 		at := urlProp.At(i)
+
+		// See if it's a plain URI.
 		if at.IsXMLSchemaAnyURI() {
 			u := at.GetXMLSchemaAnyURI()
 			urls = append(urls, u)
+			continue
+		}
+
+		// See if it's a Link obj
+		// with an href property.
+		if at.IsActivityStreamsLink() {
+			l := at.GetActivityStreamsLink()
+			hr := l.GetActivityStreamsHref()
+			if hr == nil {
+				// No href.
+				continue
+			}
+
+			if hr.IsXMLSchemaAnyURI() {
+				u := hr.Get()
+				urls = append(urls, u)
+			} else if hr.IsIRI() {
+				u := hr.GetIRI()
+				urls = append(urls, u)
+			}
 		}
 	}
 	return urls
@@ -203,6 +226,36 @@ func AppendObjectIRIs(with WithObject, object ...*url.URL) {
 	}, object...)
 }
 
+// AppendInstrumentIRIs appends the given IRIs to the Instrument property of 'with'.
+func AppendInstrumentIRIs(with WithInstrument, instrument ...*url.URL) {
+	appendIRIs(func() Property[vocab.ActivityStreamsInstrumentPropertyIterator] {
+		instrumentProp := with.GetActivityStreamsInstrument()
+		if instrumentProp == nil {
+			instrumentProp = streams.NewActivityStreamsInstrumentProperty()
+			with.SetActivityStreamsInstrument(instrumentProp)
+		}
+		return instrumentProp
+	}, instrument...)
+}
+
+// GetResultIRIs returns the IRIs contained in the `result` property of 'with'.
+func GetResultIRIs(with WithResult) []*url.URL {
+	resultProp := with.GetActivityStreamsResult()
+	return extractIRIs(resultProp)
+}
+
+// AppendResultIRIs appends the given IRIs to the Result property of 'with'.
+func AppendResultIRIs(with WithResult, result ...*url.URL) {
+	appendIRIs(func() Property[vocab.ActivityStreamsResultPropertyIterator] {
+		resultProp := with.GetActivityStreamsResult()
+		if resultProp == nil {
+			resultProp = streams.NewActivityStreamsResultProperty()
+			with.SetActivityStreamsResult(resultProp)
+		}
+		return resultProp
+	}, result...)
+}
+
 // GetTargetIRIs returns the IRIs contained in the Target property of 'with'.
 func GetTargetIRIs(with WithTarget) []*url.URL {
 	targetProp := with.GetActivityStreamsTarget()
@@ -239,6 +292,42 @@ func AppendAttributedTo(with WithAttributedTo, attribTo ...*url.URL) {
 	}, attribTo...)
 }
 
+// GetInteractingObject returns IRIs contained in the interactingObject property of 'with'.
+func GetInteractingObject(with WithInteractingObject) []*url.URL {
+	intObjProp := with.GetGoToSocialInteractingObject()
+	return getIRIs(intObjProp)
+}
+
+// AppendInteractingObject appends the given IRIs to the interactingObject property of 'with'.
+func AppendInteractingObject(with WithInteractingObject, interactingObject ...*url.URL) {
+	appendIRIs(func() Property[vocab.GoToSocialInteractingObjectPropertyIterator] {
+		intObjProp := with.GetGoToSocialInteractingObject()
+		if intObjProp == nil {
+			intObjProp = streams.NewGoToSocialInteractingObjectProperty()
+			with.SetGoToSocialInteractingObject(intObjProp)
+		}
+		return intObjProp
+	}, interactingObject...)
+}
+
+// GetInteractionTarget returns IRIs contained in the interactionTarget property of 'with'.
+func GetInteractionTarget(with WithInteractionTarget) []*url.URL {
+	intTargetProp := with.GetGoToSocialInteractionTarget()
+	return getIRIs(intTargetProp)
+}
+
+// AppendInteractionTarget appends the given IRIs to the interactionTarget property of 'with'.
+func AppendInteractionTarget(with WithInteractionTarget, interactionTarget ...*url.URL) {
+	appendIRIs(func() Property[vocab.GoToSocialInteractionTargetPropertyIterator] {
+		intTargetProp := with.GetGoToSocialInteractionTarget()
+		if intTargetProp == nil {
+			intTargetProp = streams.NewGoToSocialInteractionTargetProperty()
+			with.SetGoToSocialInteractionTarget(intTargetProp)
+		}
+		return intTargetProp
+	}, interactionTarget...)
+}
+
 // GetInReplyTo returns the IRIs contained in the InReplyTo property of 'with'.
 func GetInReplyTo(with WithInReplyTo) []*url.URL {
 	replyProp := with.GetActivityStreamsInReplyTo()
@@ -408,6 +497,25 @@ func SetPublished(with WithPublished, published time.Time) {
 	publishProp.Set(published)
 }
 
+// GetUpdated returns the time contained in the Updated property of 'with'.
+func GetUpdated(with WithUpdated) time.Time {
+	updateProp := with.GetActivityStreamsUpdated()
+	if updateProp == nil || !updateProp.IsXMLSchemaDateTime() {
+		return time.Time{}
+	}
+	return updateProp.Get()
+}
+
+// SetUpdated sets the given time on the Updated property of 'with'.
+func SetUpdated(with WithUpdated, updated time.Time) {
+	updateProp := with.GetActivityStreamsUpdated()
+	if updateProp == nil {
+		updateProp = streams.NewActivityStreamsUpdatedProperty()
+		with.SetActivityStreamsUpdated(updateProp)
+	}
+	updateProp.Set(updated)
+}
+
 // GetEndTime returns the time contained in the EndTime property of 'with'.
 func GetEndTime(with WithEndTime) time.Time {
 	endTimeProp := with.GetActivityStreamsEndTime()
@@ -501,11 +609,11 @@ func SetDiscoverable(with WithDiscoverable, discoverable bool) {
 
 // GetManuallyApprovesFollowers returns the boolean contained in the ManuallyApprovesFollowers property of 'with'.
 //
-// Returns default 'true' if property unusable or not set.
+// Returns default 'false' if property unusable or not set.
 func GetManuallyApprovesFollowers(with WithManuallyApprovesFollowers) bool {
 	mafProp := with.GetActivityStreamsManuallyApprovesFollowers()
 	if mafProp == nil || !mafProp.IsXMLSchemaBoolean() {
-		return true
+		return false
 	}
 	return mafProp.Get()
 }
@@ -520,6 +628,240 @@ func SetManuallyApprovesFollowers(with WithManuallyApprovesFollowers, manuallyAp
 	mafProp.Set(manuallyApprovesFollowers)
 }
 
+// GetHidesToPublicFromUnauthedWeb returns the boolean contained in the hidesToPublicFromUnauthedWeb property of 'with'.
+//
+// Returns default 'false' if property unusable or not set.
+func GetHidesToPublicFromUnauthedWeb(with WithHidesToPublicFromUnauthedWeb) bool {
+	hidesProp := with.GetGoToSocialHidesToPublicFromUnauthedWeb()
+	if hidesProp == nil || !hidesProp.IsXMLSchemaBoolean() {
+		return false
+	}
+	return hidesProp.Get()
+}
+
+// SetHidesToPublicFromUnauthedWeb sets the given boolean on the hidesToPublicFromUnauthedWeb property of 'with'.
+func SetHidesToPublicFromUnauthedWeb(with WithHidesToPublicFromUnauthedWeb, hidesToPublicFromUnauthedWeb bool) {
+	hidesProp := with.GetGoToSocialHidesToPublicFromUnauthedWeb()
+	if hidesProp == nil {
+		hidesProp = streams.NewGoToSocialHidesToPublicFromUnauthedWebProperty()
+		with.SetGoToSocialHidesToPublicFromUnauthedWeb(hidesProp)
+	}
+	hidesProp.Set(hidesToPublicFromUnauthedWeb)
+}
+
+// GetHidesCcPublicFromUnauthedWeb returns the boolean contained in the hidesCcPublicFromUnauthedWeb property of 'with'.
+//
+// Returns default 'true' if property unusable or not set.
+func GetHidesCcPublicFromUnauthedWeb(with WithHidesCcPublicFromUnauthedWeb) bool {
+	hidesProp := with.GetGoToSocialHidesCcPublicFromUnauthedWeb()
+	if hidesProp == nil || !hidesProp.IsXMLSchemaBoolean() {
+		return true
+	}
+	return hidesProp.Get()
+}
+
+// SetHidesCcPublicFromUnauthedWeb sets the given boolean on the hidesCcPublicFromUnauthedWeb property of 'with'.
+func SetHidesCcPublicFromUnauthedWeb(with WithHidesCcPublicFromUnauthedWeb, hidesCcPublicFromUnauthedWeb bool) {
+	hidesProp := with.GetGoToSocialHidesCcPublicFromUnauthedWeb()
+	if hidesProp == nil {
+		hidesProp = streams.NewGoToSocialHidesCcPublicFromUnauthedWebProperty()
+		with.SetGoToSocialHidesCcPublicFromUnauthedWeb(hidesProp)
+	}
+	hidesProp.Set(hidesCcPublicFromUnauthedWeb)
+}
+
+// GetApprovedBy returns the URL contained in
+// the ApprovedBy property of 'with', if set.
+func GetApprovedBy(with WithApprovedBy) *url.URL {
+	abProp := with.GetGoToSocialApprovedBy()
+	if abProp == nil || !abProp.IsIRI() {
+		return nil
+	}
+	return abProp.Get()
+}
+
+// SetApprovedBy sets the given url
+// on the ApprovedBy property of 'with'.
+func SetApprovedBy(with WithApprovedBy, approvedBy *url.URL) {
+	abProp := with.GetGoToSocialApprovedBy()
+	if abProp == nil {
+		abProp = streams.NewGoToSocialApprovedByProperty()
+		with.SetGoToSocialApprovedBy(abProp)
+	}
+	abProp.Set(approvedBy)
+}
+
+// GetLikeAuthorization returns the URL contained in
+// the likeAuthorization property of 'with', if set.
+func GetLikeAuthorization(with WithLikeAuthorization) *url.URL {
+	laProp := with.GetGoToSocialLikeAuthorization()
+	if laProp == nil || !laProp.IsIRI() {
+		return nil
+	}
+	return laProp.Get()
+}
+
+// SetLikeAuthorization sets the given url on
+// the 'likeAuthorization' property of 'with'.
+func SetLikeAuthorization(with WithLikeAuthorization, likeAuthorization *url.URL) {
+	laProp := with.GetGoToSocialLikeAuthorization()
+	if laProp == nil {
+		laProp = streams.NewGoToSocialLikeAuthorizationProperty()
+		with.SetGoToSocialLikeAuthorization(laProp)
+	}
+	laProp.Set(likeAuthorization)
+}
+
+// GetReplyAuthorization returns the URL contained in
+// the replyAuthorization property of 'with', if set.
+func GetReplyAuthorization(with WithReplyAuthorization) *url.URL {
+	raProp := with.GetGoToSocialReplyAuthorization()
+	if raProp == nil || !raProp.IsIRI() {
+		return nil
+	}
+	return raProp.Get()
+}
+
+// SetReplyAuthorization sets the given url on
+// the 'replyAuthorization' property of 'with'.
+func SetReplyAuthorization(with WithReplyAuthorization, replyAuthorization *url.URL) {
+	raProp := with.GetGoToSocialReplyAuthorization()
+	if raProp == nil {
+		raProp = streams.NewGoToSocialReplyAuthorizationProperty()
+		with.SetGoToSocialReplyAuthorization(raProp)
+	}
+	raProp.Set(replyAuthorization)
+}
+
+// GetAnnounceAuthorization returns the URL contained in
+// the announceAuthorization property of 'with', if set.
+func GetAnnounceAuthorization(with WithAnnounceAuthorization) *url.URL {
+	aaProp := with.GetGoToSocialAnnounceAuthorization()
+	if aaProp == nil || !aaProp.IsIRI() {
+		return nil
+	}
+	return aaProp.Get()
+}
+
+// SetAnnounceAuthorization sets the given url on
+// the 'announceAuthorization' property of 'with'.
+func SetAnnounceAuthorization(with WithAnnounceAuthorization, announceAuthorization *url.URL) {
+	aaProp := with.GetGoToSocialAnnounceAuthorization()
+	if aaProp == nil {
+		aaProp = streams.NewGoToSocialAnnounceAuthorizationProperty()
+		with.SetGoToSocialAnnounceAuthorization(aaProp)
+	}
+	aaProp.Set(announceAuthorization)
+}
+
+// GetMediaType returns the string contained in
+// the MediaType property of 'with', if set.
+func GetMediaType(with WithMediaType) string {
+	mtProp := with.GetActivityStreamsMediaType()
+	if mtProp == nil || !mtProp.IsRFCRfc2045() {
+		return ""
+	}
+	return mtProp.Get()
+}
+
+// SetMediaType sets the given string
+// on the MediaType property of 'with'.
+func SetMediaType(with WithMediaType, mediaType string) {
+	mtProp := with.GetActivityStreamsMediaType()
+	if mtProp == nil {
+		mtProp = streams.NewActivityStreamsMediaTypeProperty()
+		with.SetActivityStreamsMediaType(mtProp)
+	}
+	mtProp.Set(mediaType)
+}
+
+// AppendName appends the given name
+// vals to the Name property of 'with'.
+func AppendName(with WithName, name ...string) {
+	if len(name) == 0 {
+		return
+	}
+	nameProp := with.GetActivityStreamsName()
+	if nameProp == nil {
+		nameProp = streams.NewActivityStreamsNameProperty()
+		with.SetActivityStreamsName(nameProp)
+	}
+	for _, name := range name {
+		nameProp.AppendXMLSchemaString(name)
+	}
+}
+
+// AppendSummary appends the given summary
+// vals to the Summary property of 'with'.
+func AppendSummary(with WithSummary, summary ...string) {
+	if len(summary) == 0 {
+		return
+	}
+	summaryProp := with.GetActivityStreamsSummary()
+	if summaryProp == nil {
+		summaryProp = streams.NewActivityStreamsSummaryProperty()
+		with.SetActivityStreamsSummary(summaryProp)
+	}
+	for _, summary := range summary {
+		summaryProp.AppendXMLSchemaString(summary)
+	}
+}
+
+// SetBlurhash sets the given string
+// on the Blurhash property of 'with'.
+func SetBlurhash(with WithBlurhash, mediaType string) {
+	bProp := with.GetTootBlurhash()
+	if bProp == nil {
+		bProp = streams.NewTootBlurhashProperty()
+		with.SetTootBlurhash(bProp)
+	}
+	bProp.Set(mediaType)
+}
+
+// AppendSensitive appends the given sensitive
+// boolean to the `sensitive` property of 'with'.
+func AppendSensitive(with WithSensitive, sensitive bool) {
+	sProp := with.GetActivityStreamsSensitive()
+	if sProp == nil {
+		sProp = streams.NewActivityStreamsSensitiveProperty()
+		with.SetActivityStreamsSensitive(sProp)
+	}
+	sProp.AppendXMLSchemaBoolean(sensitive)
+}
+
+// AppendContent appends the given content
+// string to the `content` property of 'with'.
+func AppendContent(with WithContent, content string) {
+	cProp := with.GetActivityStreamsContent()
+	if cProp == nil {
+		cProp = streams.NewActivityStreamsContentProperty()
+		with.SetActivityStreamsContent(cProp)
+	}
+	cProp.AppendXMLSchemaString(content)
+}
+
+// AppendContentMap appends the given content
+// language map to the `content` property of 'with'.
+func AppendContentMap(with WithContent, contentMap map[string]string) {
+	cProp := with.GetActivityStreamsContent()
+	if cProp == nil {
+		cProp = streams.NewActivityStreamsContentProperty()
+		with.SetActivityStreamsContent(cProp)
+	}
+	cProp.AppendRDFLangString(contentMap)
+}
+
+// SetReplies sets the given replies collection
+// to the `replies` property of 'with'.
+func SetReplies(with WithReplies, replies vocab.ActivityStreamsCollection) {
+	rProp := with.GetActivityStreamsReplies()
+	if rProp == nil {
+		rProp = streams.NewActivityStreamsRepliesProperty()
+		with.SetActivityStreamsReplies(rProp)
+	}
+	rProp.SetActivityStreamsCollection(replies)
+}
+
 // extractIRIs extracts just the AP IRIs from an iterable
 // property that may contain types (with IRIs) or just IRIs.
 //
