@@ -26,12 +26,15 @@ const (
 type DB interface {
 	Account
 	Admin
+	AdvancedMigration
 	Application
 	Basic
+	Conversation
 	Domain
 	Emoji
 	HeaderFilter
 	Instance
+	Interaction
 	Filter
 	List
 	Marker
@@ -43,14 +46,19 @@ type DB interface {
 	Relationship
 	Report
 	Rule
+	ScheduledStatus
 	Search
 	Session
+	SinBinStatus
 	Status
 	StatusBookmark
+	StatusEdit
 	StatusFave
 	Tag
 	Thread
 	Timeline
 	User
 	Tombstone
+	WebPush
+	WorkerTask
 }
