@@ -30,7 +30,7 @@ var (
 	// ErrAlreadyExists is returned when a conflict was encountered in the db when doing an insert.
 	ErrAlreadyExists = errors.New("already exists")
 
-	// ErrBusyTimeout is returned if the database connection indicates the connection is too busy
-	// to complete the supplied query. This is generally intended to be handled internally by the DB.
-	ErrBusyTimeout = errors.New("busy timeout")
+	// ErrMultipleEntries is returned when multiple entries
+	// are found in the db when only one entry is sought.
+	ErrMultipleEntries = errors.New("multiple entries")
 )
