@@ -19,35 +19,15 @@ package log
 
 import (
 	"runtime"
-	"strings"
+
+	"codeberg.org/gruf/go-caller"
 )
 
 // Caller fetches the calling function name, skipping 'depth'.
+//
+//go:noinline
 func Caller(depth int) string {
-	var pcs [1]uintptr
-
-	// Fetch calling function using calldepth
-	_ = runtime.Callers(depth, pcs[:])
-	fn := runtime.FuncForPC(pcs[0])
-
-	if fn == nil {
-		return ""
-	}
-
-	// Get func name.
-	name := fn.Name()
-
-	// Drop all but the package name and function name, no mod path
-	if idx := strings.LastIndex(name, "/"); idx >= 0 {
-		name = name[idx+1:]
-	}
-
-	const params = `[...]`
-
-	// Drop any generic type parameter markers
-	if idx := strings.Index(name, params); idx >= 0 {
-		name = name[:idx] + name[idx+len(params):]
-	}
-
-	return name
+	pcs := make([]uintptr, 1)
+	_ = runtime.Callers(depth, pcs)
+	return caller.Get(pcs[0])
 }
