@@ -871,6 +871,20 @@ func (p *clientAPI) UndoFollow(ctx context.Context, cMsg *messages.FromClientAPI
 			follow.TargetAccountID,
 			follow.AccountID,
 		)
+
+		// Clear any notifications that were
+		// generated by this follow (request).
+		if err := p.state.DB.DeleteNotifications(
+			ctx,
+			[]gtsmodel.NotificationType{
+				gtsmodel.NotificationFollow,
+				gtsmodel.NotificationFollowRequest,
+			},
+			follow.TargetAccountID,
+			follow.AccountID,
+		); err != nil {
+			return gtserror.Newf("db error deleting notifications: %w", err)
+		}
 	}
 
 	if err := p.federate.UndoFollow(ctx, follow); err != nil {
@@ -916,10 +930,6 @@ func (p *clientAPI) UndoAnnounce(ctx context.Context, cMsg *messages.FromClientA
 		return gtserror.Newf("%T not parseable as *gtsmodel.Status", cMsg.GTSModel)
 	}
 
-	if err := p.state.DB.DeleteStatusByID(ctx, status.ID); err != nil {
-		return gtserror.Newf("db error deleting status: %w", err)
-	}
-
 	// Delete the boost wrapper status from timelines.
 	p.surface.deleteStatusFromTimelines(ctx, status.ID)
 
