@@ -473,6 +473,20 @@ func (p *fediAPI) CreateReplyRequest(ctx context.Context, fMsg *messages.FromFed
 		return gtserror.Newf("db error updating interaction request: %w", err)
 	}
 
+	// Mark the reply as now approved, referring to
+	// the accepted interaction request we just stored.
+	reply.PreApproved = false
+	reply.PendingApproval = util.Ptr(false)
+	reply.ApprovedByURI = req.AuthorizationURI
+	if err := p.state.DB.UpdateStatus(
+		ctx,
+		reply,
+		"pending_approval",
+		"approved_by_uri",
+	); err != nil {
+		return gtserror.Newf("db error updating status: %w", err)
+	}
+
 	// Send out the accept.
 	if err := p.federate.AcceptInteraction(ctx, req); err != nil {
 		log.Errorf(ctx, "error federating accept: %v", err)
@@ -756,17 +770,11 @@ func (p *fediAPI) CreateLikeRequest(ctx context.Context, fMsg *messages.FromFedi
 		return gtserror.Newf("db error updating interaction request: %w", err)
 	}
 
-	// Send out the accept.
-	if err := p.federate.AcceptInteraction(ctx, req); err != nil {
-		log.Errorf(ctx, "error federating accept: %v", err)
-	}
-
-	// Mark the fave as approved.
+	// Mark the status fave as now approved, referring to
+	// the accepted interaction request we just stored.
+	req.Like.PreApproved = false
 	req.Like.PendingApproval = util.Ptr(false)
 	req.Like.ApprovedByURI = req.AuthorizationURI
-	req.Like.PreApproved = false
-
-	// Update in the db.
 	if err := p.state.DB.UpdateStatusFave(
 		ctx,
 		req.Like,
@@ -776,6 +784,11 @@ func (p *fediAPI) CreateLikeRequest(ctx context.Context, fMsg *messages.FromFedi
 		return gtserror.Newf("db error updating status fave: %w", err)
 	}
 
+	// Send out the accept.
+	if err := p.federate.AcceptInteraction(ctx, req); err != nil {
+		log.Errorf(ctx, "error federating accept: %v", err)
+	}
+
 	// Notify the faved account.
 	if err := p.surface.notifyFave(ctx, req.Like); err != nil {
 		log.Errorf(ctx, "error notifying fave: %v", err)
@@ -979,6 +992,20 @@ func (p *fediAPI) CreateAnnounceRequest(ctx context.Context, fMsg *messages.From
 		return gtserror.Newf("db error updating interaction request: %w", err)
 	}
 
+	// Mark the boost as now approved, referring to
+	// the accepted interaction request we just stored.
+	boost.PreApproved = false
+	boost.PendingApproval = util.Ptr(false)
+	boost.ApprovedByURI = req.AuthorizationURI
+	if err := p.state.DB.UpdateStatus(
+		ctx,
+		boost,
+		"pending_approval",
+		"approved_by_uri",
+	); err != nil {
+		return gtserror.Newf("db error updating status: %w", err)
+	}
+
 	// Send out the accept.
 	if err := p.federate.AcceptInteraction(ctx, req); err != nil {
 		log.Errorf(ctx, "error federating accept: %v", err)
@@ -1548,6 +1575,20 @@ func (p *fediAPI) UndoFollow(ctx context.Context, fMsg *messages.FromFediAPI) er
 		return gtserror.Newf("%T not parseable as *gtsmodel.Follow", fMsg.GTSModel)
 	}
 
+	// Clear any notifications that were
+	// generated by this follow (request).
+	if err := p.state.DB.DeleteNotifications(
+		ctx,
+		[]gtsmodel.NotificationType{
+			gtsmodel.NotificationFollow,
+			gtsmodel.NotificationFollowRequest,
+		},
+		follow.TargetAccountID,
+		follow.AccountID,
+	); err != nil {
+		return gtserror.Newf("db error deleting notifications: %w", err)
+	}
+
 	if follow.Account.IsLocal() {
 		// Remove posts by target from origin's timelines.
 		p.surface.removeRelationshipFromTimelines(ctx,
